package com.gic.enterprise.service.outer.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.enterprise.dto.data.*;
import com.gic.enterprise.entity.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;

@Service("dataConfigApiService")
public class DataConfigApiServiceImpl implements DataConfigApiService {
    @Autowired
    private DataMemberConsumeConfigService dataMemberConsumeConfigService;
    @Autowired
    private DataWechatMemberConfigService dataWechatMemberConfigService;
    @Autowired
    private DataActuallyPaidConfigService dataActuallyPaidConfigService;
    @Autowired
    private DataSingleEffectConfigService dataSingleEffectConfigService;
    @Autowired
    private DataPerformanceConfigService dataPerformanceConfigService;
    @Autowired
    private DataGenerationConfigService dataGenerationConfigService;

    @Override
    public ServiceResponse<Void> configWechatMember(DataWechatMemberConfigDTO dto) {
        String memberCardId = dto.getMemberCardArea();
        if (StringUtils.isBlank(memberCardId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "会员卡域不能为空");
        }
        TabDataWechatMemberConfig result = dataWechatMemberConfigService.getByMemberCardId(dto.getEnterpriseId(), memberCardId);

        //是否是刷新通道的保存
        boolean isRefresh = dto.getValidRefresh() == 1;
        if (isRefresh) {
            dto.setValidSave(0);
        } else {
            dto.setValidSave(1);
        }
        if (result == null) {
            dataWechatMemberConfigService.saveWechatMemberConfig(dto);
        } else {
            dto.setWechatMemberConfigId(result.getWechatMemberConfigId());
            dataWechatMemberConfigService.editWechatMemberConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> newWechatMember(Integer enterpriseId, Long memberCardArea) {
        return configWechatMember(new DataWechatMemberConfigDTO().setEnterpriseId(enterpriseId).setMemberCardArea(memberCardArea.toString()).setValidRefresh(1));
    }

    @Override
    public ServiceResponse<Void> deleteWechatMember(Integer enterpriseId, Long memberCardArea) {
        if (memberCardArea == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "会员卡域不能为空");
        }
        TabDataWechatMemberConfig result = dataWechatMemberConfigService.getByMemberCardId(enterpriseId, memberCardArea.toString());
        if (result != null) {
            dataWechatMemberConfigService.editWechatMemberConfig(new DataWechatMemberConfigDTO()
                    .setWechatMemberConfigId(result.getWechatMemberConfigId())
                    .setStatus(0));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataWechatMemberConfigDTO>> listWechatMemberConfig(Integer enterpriseId) {
        return ServiceResponse.success(Optional
                .ofNullable(EntityUtil.changeEntityListNew(DataWechatMemberConfigDTO.class, dataWechatMemberConfigService
                        .listByEnterpriseId(enterpriseId))).orElse(Collections.EMPTY_LIST));
    }

    @Override
    public ServiceResponse<Integer> configMemberConsume(DataMemberConsumeConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        TabDataMemberConsumeConfig record = dataMemberConsumeConfigService.getByEnterpriseId(dto.getEnterpriseId());
        if (record == null) {
            //还未配置，新增
            return ServiceResponse.success(dataMemberConsumeConfigService.saveMemberConsumeConfig(dto));
        }
        dto.setMemberConsumeConfigId(record.getMemberConsumeConfigId());
        dataMemberConsumeConfigService.editMemberConsumeConfig(dto);
        return ServiceResponse.success(record.getMemberConsumeConfigId());
    }

    @Override
    public ServiceResponse<DataMemberConsumeConfigDTO> getMemberConsumeConfigByEnterpriseId(Integer enterpriseId) {
        TabDataMemberConsumeConfig record = dataMemberConsumeConfigService.getByEnterpriseId(enterpriseId);
        if (record == null) {
            TabDataMemberConsumeConfig defaultRecord = dataMemberConsumeConfigService.getByEnterpriseId(-1);
            return ServiceResponse.success(EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, defaultRecord));
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initMemberConsumeConfig(Integer enterpriseId) {
        TabDataMemberConsumeConfig defaultRecord = dataMemberConsumeConfigService.getByEnterpriseId(-1);
        DataMemberConsumeConfigDTO dto = EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, defaultRecord);
        dto.setMemberConsumeConfigId(null);
        dto.setEnterpriseId(enterpriseId);
        configMemberConsume(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configActuallyPaid(List<DataActuallyPaidConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "实付配置内容为空");
        }
        for (DataActuallyPaidConfigDTO dto : dtoList) {
            if (dto.getClassify() == null) {
                dto.setClassify(1);
            }
            TabDataActuallyPaidConfig temp = dataActuallyPaidConfigService.get(dto.getEnterpriseId(), dto.getClassify());
            if (temp != null && temp.getHasConfig() == 1) {
                //return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已经配置过，不允许再次配置");
                continue;
            }
            if (temp == null) {
                //如果是第一次生成，则
                dto.setHasConfig(0);
            } else {
                dto.setHasConfig(1);
            }
            dataActuallyPaidConfigService.delete(dto.getEnterpriseId(), dto.getClassify());
            dto.setConfigStatus(Optional.ofNullable(dto.getConfigStatus()).orElse(0));
            dataActuallyPaidConfigService.saveActuallyPaidConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configActuallyPaid(Integer enterpriseId, Integer memberBusiness, Integer performanceCount, Integer memberBusinessOfGoods) {
        List<DataActuallyPaidConfigDTO> list = new ArrayList<>(5);
        boolean isRight = enterpriseId != null && memberBusiness != null && performanceCount != null && memberBusinessOfGoods != null;
        if (!isRight) {
            ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数不能为空");
        }
        if (memberBusiness != null) {
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(1)
                    .setConfigStatus(memberBusiness));
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(2)
                    .setConfigStatus(memberBusiness));
        }
        if (performanceCount != null) {
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(3)
                    .setConfigStatus(performanceCount));
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(4)
                    .setConfigStatus(performanceCount));
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(5)
                    .setConfigStatus(performanceCount));
        }
        if (memberBusinessOfGoods != null) {
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(6)
                    .setConfigStatus(memberBusinessOfGoods));
        }

        return configActuallyPaid(list);
    }

    @Override
    public ServiceResponse<List<DataActuallyPaidConfigDTO>> listActuallyPaidConfig(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(enterpriseId);
        if (CollectionUtils.isEmpty(list)) {
            //如果没有配置，默认数据
            return ServiceResponse.success(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class,
                    dataActuallyPaidConfigService.listByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class, list));
    }

    @Override
    public ServiceResponse<DataActuallyPaidConfig> getDataActuallyPaidConfig(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            Map<String, TabDataActuallyPaidConfig> map = list.stream().collect(Collectors.toMap(e -> e.getClassify().toString(), e -> e));
            //如果没有配置，默认数据
            Integer memberBusiness = map.get("1").getConfigStatus();
            Integer performanceCount = map.get("3").getConfigStatus();
            Integer memberBusinessOfGoods = map.get("6").getConfigStatus();
            return ServiceResponse.success(new DataActuallyPaidConfig()
                    .setEnterpriseId(enterpriseId)
                    .setMemberBusiness(memberBusiness)
                    .setPerformanceCount(performanceCount)
                    .setHasConfigMemberBusiness(map.get("1").getHasConfig() == 1)
                    .setHasConfigPerformanceCount(map.get("3").getHasConfig() == 1)
                    .setHasConfigMemberBusinessOfGoods(map.get("6").getHasConfig() == 1)
                    .setMemberBusinessOfGoods(memberBusinessOfGoods));
        }
        return ServiceResponse.success(new DataActuallyPaidConfig().setEnterpriseId(enterpriseId).setMemberBusiness(0).setPerformanceCount(0).setMemberBusinessOfGoods(0));
    }

    @Override
    public ServiceResponse<Void> initActuallyPaidData(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(-1);
        list = list.stream().map(e -> e.setEnterpriseId(enterpriseId)).map(e -> e.setActuallyPaidConfigId(null))
                .collect(Collectors.toList());
        configActuallyPaid(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class, list));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configSingleEffect(List<DataSingleEffectConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "单效配置内容为空");
        }
        for (DataSingleEffectConfigDTO dto : dtoList) {
            dataSingleEffectConfigService.deleteSingleEffectConfig(dto.getEnterpriseId(), dto.getOrderGrain());
            dataSingleEffectConfigService.saveSingleEffectConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataSingleEffectConfigDTO>> getSingleEffectConfig(Integer enterpriseId) {
        List<TabDataSingleEffectConfig> record = dataSingleEffectConfigService.getByEnterpriseId(enterpriseId);
        if (CollectionUtils.isEmpty(record)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class,
                    dataSingleEffectConfigService.getByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initSingleEffectConfig(Integer enterpriseId) {
        List<DataSingleEffectConfigDTO> dtoList = EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class,
                dataSingleEffectConfigService.getByEnterpriseId(-1));
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId))
                .map(e -> e.setSingleEffectConfigId(null)).collect(Collectors.toList());
        configSingleEffect(dtoList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configPerformance(DataPerformanceConfigDTO dto) {
        TabDataPerformanceConfig record = dataPerformanceConfigService.getByEnterpriseId(dto.getEnterpriseId());
        if (record == null) {
            dataPerformanceConfigService.savePerformanceConfig(dto);
            return ServiceResponse.success();
        }
        dto.setPerformanceConfigId(record.getPerformanceConfigId());
        dataPerformanceConfigService.editPerformanceConfig(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<DataPerformanceConfigDTO> getPerformanceConfig(Integer enterpriseId) {
        TabDataPerformanceConfig record = dataPerformanceConfigService.getByEnterpriseId(enterpriseId);
        if (record == null) {
            return ServiceResponse.success(EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class,
                    dataPerformanceConfigService.getByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initPerformanceConfig(Integer enterpriseId) {
        DataPerformanceConfigDTO dto = EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class,
                dataPerformanceConfigService.getByEnterpriseId(-1));
        dto.setPerformanceConfigId(null);
        dto.setEnterpriseId(enterpriseId);
        configPerformance(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveGenerationConfig(List<DataGenerationConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "年龄段配置内容为空");
        }
        dataGenerationConfigService.deleteGenerationConfig(dtoList.get(0).getEnterpriseId());
        dtoList.stream()
                .forEach(e -> dataGenerationConfigService.saveGenerationConfig(e));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataGenerationConfigDTO>> listGenerationConfig(Integer enterpriseId) {
        List<TabDataGenerationConfig> list = dataGenerationConfigService.listGenerationByEnterpriseId(enterpriseId);
        return ServiceResponse.success(CollectionUtils.isEmpty(list) ?
                EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class, dataGenerationConfigService.listGenerationByEnterpriseId(-1)) :
        EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> initGenerationConfig(Integer enterpriseId) {
        List<DataGenerationConfigDTO> dtoList = EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class,
                dataGenerationConfigService.listGenerationByEnterpriseId(-1));
        saveGenerationConfig(dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId))
                .map(e -> e.setGenerationConfigId(null)).collect(Collectors.toList()));
        return ServiceResponse.success();
    }
}
